# Generated by roxygen2: do not edit by hand

export(fdt)
export(fdt_locales_lst)
export(fdt_locales_vec)
export(first_day_of_week)
export(flex_d_lst)
export(flex_d_vec)
export(flex_t12_lst)
export(flex_t12_vec)
export(flex_t24_lst)
export(flex_t24_vec)
export(names_months)
export(names_wkdays)
export(standard_date)
export(standard_date_time)
export(standard_time)
