% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fdf_ftf.R
\name{flex_t24_vec}
\alias{flex_t24_vec}
\title{Get a vector of all 24-hour flexible time types}
\usage{
flex_t24_vec()
}
\value{
A character vector of supported 24-hour flexible time types.
}
\description{
The \code{flex_t24_vec()} function produces a vector of all supported flexible
24-hour time types in the \strong{bigD} package. These types are essentially
identifiers for classes of cross-locale time formatting, so, none of these
should be used directly in the \code{format} argument of the \code{fdt()} function (use
the \link{flex_t24_lst} object for that).
}
